#! /bin/sh
#
# Copyright by The HDF Group.
# All rights reserved.
#
# Tests for the HDF NPP aggregation tool

# The build (current) directory might be different than the source directory.
if test -z "$srcdir"; then
    srcdir=.
fi

# Include Nagg test functions.
. $srcdir/nagg_functions.sh

# Main body goes here.
dir=$SRC_TOOL_TESTFILES
#======================================#
# Positive tests: Expected to succeed. #
# Exit code is zero.                   #
#======================================#
echo ============================================
echo The following tests are expected to succeed.
echo ============================================
# tests expected to succeed (exit code is 0)
CHECK -h
CHECK_OUTPUT usage.txt --help
# Test --debug option before using it for the remaining tests.
# Check it can show correct output.
CHECK --debug --help
# Check --version
CHECK_VERSION
CHECK --version

# Test real data files
# REDRO files each has 16 granules. Test if nagg can deaggregate them.
dfile=$dir/REDRO_npp_d20030125_t0702533_e0711257_b00014_c20111025170507190780_unkn_ada.h5
# Nagg should be able to find the geolocation file. Leave it here for reference.
#geofile=GCRIO_npp_d20030125_t0702533_e0711257_b00014_c20111025170507184515_unkn_ada.h5
CHECK_OUTPUT redro1.txt -t REDRO $dfile
CHECK_OUTPUT redro3.txt -t REDRO -n 3 $dfile
# The granule duration of a REDRO product is 31.997 seconds.
# Therefore -A 95 == -n 3 while -A 96 == -n 4.
CHECK_OUTPUT redro3.txt -t REDRO -A 95 $dfile
CHECK_OUTPUT redro4.txt -t REDRO -A 96 $dfile
# Verify the above two cases by dumping the produced output files.
CHECK_OUTPUT redro1-dump.txt dump -t REDRO $dfile
CHECK_OUTPUT redro3-dump.txt dump -t REDRO -n 3 $dfile

# Test -d option.
# Redundent. "-d ." is same as not specifying it as where files are
# created. Output file names still contain "./" as prefix.
CHECK_OUTPUT redro1-d-redundant.txt -d . -t REDRO $dfile
# Create a temporary directory and send the output files to it.
tmpdir=tmpdir
test -e $tmpdir || mkdir $tmpdir
CHECK_OUTPUT redro1-d.txt -d $tmpdir -t REDRO $dfile
rm -rf $tmpdir

# Redo above but with -S.  Expect to create unpackaged (Simple) files.
CHECK_OUTPUT redroS.txt -S -t REDRO $dfile
CHECK_OUTPUT redro3S.txt -S -t REDRO -n 3 $dfile
# Verify that --debug shows valid external geo filenames and null geo filenames
# that are expected on output files in unpackaged format.
CHECK_OUTPUT redro3S-dump.txt dump -S -t REDRO -n 3 $dfile

# test aggregating REDRO/GCRIO files with duplicate granules of different versions.
dfile=$dir/REDRO_npp_d20120513_t000*_noaa_ops.h5
CHECK_OUTPUT redro32.txt -n32 -tREDRO $dfile

# SATMS files each has 1 granule. Test if nagg can aggregate them.
dfile="$dir/SATMS*.h5"
CHECK_OUTPUT satms8.txt -t SATMS -n 8 $dfile

# SATMS, TATMS compatible input granules with GEO GATMO unpackaged input, packaged 
# output. 
dfile="$dir/SATMS_npp_d201204*.h5 $dir/TATMS_npp_d201204*.h5"
CHECK_OUTPUT satms_tatms2.txt -n2 -t SATMS,TATMS $dfile
# Same again but switch the order of the -t list. Should have the same output.
CHECK_OUTPUT satms_tatms2.txt -n2 -t TATMS,SATMS $dfile

# SATMS, TATMS compatible input granules with GEO GATMO unpackaged input, packaged 
# output.  Missing TATMS granules are to be replaced by fill granules.
dfile="$dir/SATMS_npp_d201204*.h5 $dir/TATMS_npp_d20120404*.h5"
CHECK_OUTPUT satms_tatms2m.txt -n2 -t SATMS,TATMS $dfile
# Verify that fill granules are added accordingly.
CHECK_OUTPUT satms_tatms2m-dump.txt dump -n2 -t SATMS,TATMS $dfile

# ICSTT with non-matching geofilename using -g yes - unpackaged output
echo Test -g yes unpackaged
dfile=$dir/ICSTT_npp_d20030125_t0705359_e0713337_b00014_c20111025184111991145_unkn_ada.h5
CHECK -n 5 -t ICSTT -S -g yes $dfile

# Single product SATMS packaged with GEO GATMO - packaged output
echo Test packaged input packaged output
dfile=$dir/GATMO-SATMS_npp_d20030126_t0402304_e0411338_b00029_c20120126220649940102_devl_ops.h5
CHECK -n 7 -t SATMS $dfile

# Same again, but with unpackaged output
echo Test packaged input unpackaged output
dfile=$dir/GATMO-SATMS_npp_d20030126_t0402304_e0411338_b00029_c20120126220649940102_devl_ops.h5
CHECK -n 7 -S -t SATMS $dfile

# 4 compatible unpackaged products without GEO - packaged output 
echo Test 4 compatible products unpackaged input packaged output 
dfile="$dir/SVM04_npp_d20120229_t0530029_e0535433_b01757_c20120229113543649418_noaa_ops.h5 $dir/SVM05_npp_d20120229_t0530029_e0535433_b01757_c20120229113543649746_noaa_ops.h5 $dir/SVM06_npp_d20120229_t0530029_e0535433_b01757_c20120229113543650046_noaa_ops.h5 $dir/SVM09_npp_d20120229_t0530029_e0535433_b01757_c20120229113543650958_noaa_ops.h5"
CHECK -n2 -g no -t SVM04,SVM05,SVM06,SVM09 $dfile

# Same again, but with unpackaged output
echo Test 4 compatible products unpackaged input unpackaged output
CHECK -n2 -g no -S -t SVM04,SVM05,SVM06,SVM09 $dfile

# ICSTT with unpackaged input to produce unpackaged output without GEO
echo Test -g no unpackaged
dfile=$dir/ICSTT_npp_d20030125_t0705359_e0713337_b00014_c20111025184111991145_unkn_ada.h5
CHECK -n 5 -t ICSTT -g no $dfile

# SVI02,SVI03 no GEO, fixed dimension files, one missing granule
echo Test -g no fixed dimensions
dfile="$dir/SVI02_aqu_d20110624_t1739177_e1740275_b48626_c20110624210756057075_grav_dev.h5 $dir/SVI03_aqu_d20110624_t1739177_e1740275_b48626_c20110624210732114546_grav_dev.h5 $dir/SVI03_aqu_d20110624_t1738068_e1739166_b48626_c20110624210810775985_grav_dev.h5"
CHECK -n2 -tSVI02,SVI03 -g no $dfile 

# GATMO-SATMS packaged input and output without GEO
echo Test -g no packaged
dfile=$dir/GATMO-SATMS_npp_d20030126_t0402304_e0411338_b00029_c20120126220649940102_devl_ops.h5
CHECK -n 5 -t SATMS -g no $dfile

# GEO product alone - -g GCRIO option
echo Test -g "<geoproduct>"
dfile=$dir/GCRIO_npp_d20030125_t0702533_e0711257_b00014_c20111025170507184515_unkn_ada.h5
CHECK -n 8 -g GCRIO $dfile

# Tests to validate --like -l option.
echo "Test just -l (no -n nor -t)"
# REDRO files each has 16 granules. The -l should produce 16 granules output,
# instead of the default 1 granule.
# Nagg should be able to find the geolocation file. Leave it here for reference.
#geofile=GCRIO_npp_d20030125_t0702533_e0711257_b00014_c20111025170507184515_unkn_ada.h5
exfile=$dir/REDRO_npp_d20030125_t0702533_e0711257_b00014_c20111025170507190780_unkn_ada.h5
dfile=$dir/REDRO_npp_d20030125_t0702533_e0711257_b00014_c20111025170507190780_unkn_ada.h5
CHECK_OUTPUT redro16.txt -l $exfile $dfile
# check --like works the same
CHECK_OUTPUT redro16.txt --like $exfile $dfile
# Override -l with -n 3 (-p should not be in)
CHECK_OUTPUT redro3.txt -l $exfile -n 3 $dfile
echo "Test --like with multiple products example file"
# test --like form of option with 2 compatible products + GEO, 5 granules of each product in
# de-aggregated input files.  Produces 3 packaged output files.
exfile=$dir/GATMO-SATMS-TATMS_npp_d20120403_t2359123_e0000159_b02250_c20120830155722719166_unkn_xxx.h5
dfile="$dir/SATMS_npp_d2012040*.h5 $dir/TATMS_npp_d2012040*.h5"
CHECK_OUTPUT satms_tatms2.txt --like $exfile $dfile
# Override -l of two products with -t one product
CHECK_OUTPUT satms2.txt -t SATMS -l $exfile $dfile
# Check if -g no(0) works right with -l
CHECK_OUTPUT satms_tatms2-g0.txt -g no -l $exfile $dfile

#===================================#
# Negative tests: Expected to fail. #
# Exit code is non-zero.            #
#===================================#
echo =========================================
echo The following tests are expected to fail.
echo =========================================
# unknown option
CHECK_FAIL_OUTPUT usage-bad-option.txt -x
# Non-positive -n number of granules
CHECK_FAIL -n 0
# Missing input file argument
CHECK_FAIL -n 3

echo Test -d option - Errors expected
# missing option parameter
dfile=$dir/REDRO_npp_d20030125_t0702533_e0711257_b00014_c20111025170507190780_unkn_ada.h5
CHECK_FAIL -d -t REDRO $dfile
# output directory does not exist or not writable. Assume /tmpbad does not exist.
dfile=$dir/REDRO_npp_d20030125_t0702533_e0711257_b00014_c20111025170507190780_unkn_ada.h5
CHECK_FAIL -d /tmpbad -t REDRO $dfile
# Verify Nagg can detect if the outdir name is too long for the output file
# names buffer. Each output file name has two parts, Head and Tail.  The Tail
# size is fixed. The Head consists of the outdir, products and geolocation
# product.
# The Header part can hold (DPID_NUM_MAX+1) [31] of DPID_size[5] and "_". That
# is 31*6=186 chars.
# Outdir length test 1: outdir length is 167, '/', two products and one Geo
# are 18.  Total is 186. Should pass.
dfile="$dir/SATMS_npp_d201204*.h5 $dir/TATMS_npp_d201204*.h5"
outdir=\
123456789a123456789b123456789c123456789d123456789e123456789f123456789g123456789h123456789i123456789j123456789k123456789l123456789l123456789m123456789n123456789o1234567
test -e $outdir || mkdir $outdir
CHECK_OUTPUT satms_tatms2-d.txt -d $outdir -n 2 -t SATMS,TATMS $dfile
rm -rf $outdir
# Outdir length test 2: increase outdir length by 1 to 168.
# Total is 187. Should fail.
outdir="$outdir"8
test -e $outdir || mkdir $outdir
CHECK_FAIL -d $outdir -n 2 -t SATMS,TATMS $dfile
# Outdir length test 3: Use -g 0 to not generate Geolocation product. Total
# decreases to 181. Should pass.
CHECK_OUTPUT satms_tatms2-d-g0.txt -g 0 -d $outdir -n 2 -t SATMS,TATMS $dfile
rm -rf $outdir
# Outdir length test 4: increate outdir length by 6 to 174. With 2 produces,
# Total is 187. Should fail.
outdir="$outdir"9p1234
test -e $outdir || mkdir $outdir
CHECK_FAIL -g 0 -d $outdir -n 2 -t SATMS,TATMS $dfile
rmdir $outdir
# Outdir length test 5: Use -S (Simple==Unpackaged form).  Only 1 product per
# file. Increase outdir by 5 to 179. Total is 186. Should pass.
outdir="$outdir"56789
test -e $outdir || mkdir $outdir
CHECK_OUTPUT satms_tatms2-d-S.txt -S -d $outdir -n 2 -t SATMS,TATMS $dfile
rm -rf $outdir
# Outdir length test 6: Increse outdir by 1 to 180. Total is 187. Should fail.
outdir="$outdir"q
test -e $outdir || mkdir $outdir
CHECK_FAIL -S -d $outdir -n 2 -t SATMS,TATMS $dfile
# Outdir length test 7: Use -g 0 will not help. Still one product per file,
# just no geoproduct file. Total is still 187. Should fail.
CHECK_FAIL -g 0 -S -d $outdir -n 2 -t SATMS,TATMS $dfile

echo Test -A bad values. All should fail.
dfile=$dir/REDRO_npp_d20030125_t0702533_e0711257_b00014_c20111025170507190780_unkn_ada.h5
# -A missting parameter
CHECK_FAIL -A -t REDRO $dfile
# -A non-positive integer parameter
CHECK_FAIL -A 0 -t REDRO $dfile
CHECK_FAIL -A -1 -t REDRO $dfile
CHECK_FAIL -A  319.97 -t REDRO $dfile
CHECK_FAIL -A 32s -t REDRO $dfile
# -A and -n are exclusive
CHECK_FAIL -A 150 -n 5 -t REDRO $dfile

echo Test -g strict - Errors expected
# ICSTT with non-matching geofilename using -g strict
dfile=$dir/ICSTT_npp_d20030125_t0705359_e0713337_b00014_c20111025184111991145_unkn_ada.h5
CHECK_FAIL -n 15 -t ICSTT -g strict $dfile

echo Test -g "<geoproduct>" error checking
dfile=$dir/GCRIO_npp_d20030125_t0702533_e0711257_b00014_c20111025170507184515_unkn_ada.h5
# -t should not be used with -g geoproduct
CHECK_FAIL -n 8 -t REDRO -g GCRIO $dfile
# One of -t or -g geoproduct should be used
CHECK_FAIL -n 8 $dfile
# Input files have the wrong geoproduct
CHECK_FAIL -n 8 -g GATMO $dfile

echo Test --like -l with bad parameters. All should fail.
exfile=$dir/GATMO-SATMS-TATMS_npp_d20120403_t2359123_e0000159_b02250_c20120830155722719166_unkn_xxx.h5
dfile="$dir/SATMS_npp_d2012040*.h5 $dir/TATMS_npp_d2012040*.h5"
dfile2=$dir/REDRO_npp_d20030125_t0702533_e0711257_b00014_c20111025170507190780_unkn_ada.h5
# -l without example file.
CHECK_FAIL -l -n 2 $dfile
# -l with non-existing file
CHECK_FAIL -l nosuch -n 2 $dfile
# override with bad -t value. Example file has SATMS and TATMS products but
# -t asks for REDRO.
CHECK_FAIL -l $exfile -t REDRO $dfile
# Example file has SATMS and TATMS but datafiles have REDRO only.
CHECK_FAIL -l $exfile $dfile2
 

# Post results and exit.
echo
if test $nerrors -eq 0 ; then
    echo "All $TESTNAME tests passed."
    exit $EXIT_SUCCESS
else
    echo "$TESTNAME tests failed with $nerrors errors."
    exit $EXIT_FAILURE
fi
